"""
PyGL3Display
(C) David Griffin (2010-present)
"Habilain" is a pseudonym used by David Griffin in a number of places.

A high performance Pygame OpenGL library, capable of handling 1000s of sprites

Requirements: Python, Pygame, PyOpenGL, Open GL 2.0 graphics hardware

Licensed under GPLv2 or GPLv3. You should have received a copy of both of these
licenses in the files GPLv2.txt and GPLv3.txt. If not these licenses can be
found at http://www.gnu.org/licenses/gpl-2.0.html (GPLv2) and
http://www.gnu.org/licenses/gpl-3.0.html (GPLv3). By using this file you agree
to be bound by at least one of these licenses.

Other licenses, including commercial/proprietary licenses, can be arranged by 
contacting me via e-mail at habilain@gmail.com. If you do not have in 
writing an alternative license agreement from me, you must use one of the 
licenses specified above when releasing work which makes use of this library.

pygl3display.compute.reducers

A couple of reducers:
minMax: returns the min/max values of a greyscale texture
texSum: Sums all values in the greyscale texture
"""

from pygl3display.compute.reducer import makeReducer
minMaxSampleSrc = """
vec4 sample(in vec4 x0, in vec4 x1, in vec4 x2, in vec4 x3);
vec4 sample(in vec4 x0, in vec4 x1, in vec4 x2, in vec4 x3){
    float mx1 = (x0.x > x1.x) ? x0.x : x1.x;
    float mx2 = (x2.x > x3.x) ? x2.x : x3.x;
    float mx  = (mx1 >  mx2)  ? mx1  : mx2;
    float mn1 = (x0.y < x1.y) ? x0.y : x1.y;
    float mn2 = (x2.y < x3.y) ? x2.y : x3.y;
    float mn  = (mn1  < mn2)  ? mn1  : mn2;
    return vec4(mx, mn, 0.0, 1.0);
}
"""

minMax = makeReducer('MinMax', minMaxSampleSrc, 2)

texSumSampleSrc = """
vec4 sample(in vec4 x0, in vec4 x1, in vec4 x2, in vec4 x3);
vec4 sample(in vec4 x0, in vec4 x1, in vec4 x2, in vec4 x3){
    return vec4(x0.x + x1.x + x2.x + x3.x, 0.0, 0.0, 1.0);
}
"""

texSum = makeReducer('TexSum', texSumSampleSrc, 1)

