"""
PyGL3Display
(C) David Griffin (2010-present)
"Habilain" is a pseudonym used by David Griffin in a number of places.

A high performance Pygame OpenGL library, capable of handling 1000s of sprites

Requirements: Python, Pygame, PyOpenGL, Open GL 2.0 graphics hardware

Licensed under GPLv2 or GPLv3. You should have received a copy of both of these
licenses in the files GPLv2.txt and GPLv3.txt. If not these licenses can be
found at http://www.gnu.org/licenses/gpl-2.0.html (GPLv2) and
http://www.gnu.org/licenses/gpl-3.0.html (GPLv3). By using this file you agree
to be bound by at least one of these licenses.

Other licenses, including commercial/proprietary licenses, can be arranged by 
contacting me via e-mail at habilain@gmail.com. If you do not have in physical 
writing an alternative license agreement from me, you must use one of the 
licenses specified above when releasing work which makes use of this library.

pygl3display.gl

Provides the GL Binding of PyGL3Display. This can either be OpenGL or CFFI
implementation"""

import os
backend = None
if os.name == 'posix':
    try:
        from pygl3display.gl.cffigl import *
        backend = 'cffigl'
    except ImportError:
        import sys
        sys.stderr.write('Could not import cffigl backend; falling back to pyopengl')

if backend is None:
    from pygl3display.gl.pyopengl import *
    from pygl3display.gl.ctypesdata import *
    backend = 'pyopengl'

class TexUnitsDict(dict):
    """Maps integers to available texture units"""
    def __missing__(self, key):
        """Lazily create the map between integer i and GL_TEXTUREi"""
        if isinstance(key, int) and 0 <= key < GL_MAX_TEXTURE_COORDS:
            tuname = 'GL_TEXTURE%i' % key
            self[key] = globals()[tuname]
            return self[key]
        else:
            raise KeyError('%s is not a valid Texture Unit' % str(key))
        
class ColorAttachmentDict(dict):
    def __missing__(self, key):
        names = [key, key + 'EXT', key + '_EXT']
        lookup = globals()
        for name in names:
            if name in lookup:
                return lookup[name]
        raise Exception('Couldn\'t find %s' % key)
