"""
PyGL3Display
(C) David Griffin (2010-present)
"Habilain" is a pseudonym used by David Griffin in a number of places.

A high performance Pygame OpenGL library, capable of handling 1000s of sprites

Requirements: Python, Pygame, PyOpenGL, Open GL 2.0 graphics hardware

Licensed under GPLv2 or GPLv3. You should have received a copy of both of these
licenses in the files GPLv2.txt and GPLv3.txt. If not these licenses can be
found at http://www.gnu.org/licenses/gpl-2.0.html (GPLv2) and
http://www.gnu.org/licenses/gpl-3.0.html (GPLv3). By using this file you agree
to be bound by at least one of these licenses.

Other licenses, including commercial/proprietary licenses, can be arranged by 
contacting me via e-mail at habilain@gmail.com. If you do not have in physical 
writing an alternative license agreement from me, you must use one of the 
licenses specified above when releasing work which makes use of this library.

pygl3display.gl.cffigl.cffigl_dev5

Python port of http://www.opengl-tutorial.org/ Example 5"""

from cffigl import *
from cffigl_dev_share import *
import math

vertexShader = """ 
#version 120

// Input vertex data, different for all executions of this shader.
attribute vec3 vertexPosition_modelspace;
attribute vec2 vertexUV;

// Output data ; will be interpolated for each fragment.
varying vec2 UV;

// Values that stay constant for the whole mesh.
uniform mat4 MVP;

void main(){

    // Output position of the vertex, in clip space : MVP * position
    gl_Position =  MVP * vec4(vertexPosition_modelspace,1);
    
    // UV of the vertex. No special space for this one.
    UV = vertexUV;
}

"""

fragmentShader = """ 
#version 120

// Interpolated values from the vertex shaders
varying vec2 UV;

// Values that stay constant for the whole mesh.
uniform sampler2D myTextureSampler;

void main(){

    // Output color = color of the texture at the specified UV
    gl_FragColor = texture2D( myTextureSampler, UV );
}
"""

vdata = [-1.0,-1.0, -1.0, -1.0,-1.0, 1.0, -1.0, 1.0, 1.0,
    1.0, 1.0,-1.0, -1.0,-1.0,-1.0, -1.0, 1.0,-1.0,
    1.0,-1.0, 1.0, -1.0,-1.0,-1.0, 1.0,-1.0,-1.0,
    1.0, 1.0,-1.0, 1.0,-1.0,-1.0, -1.0,-1.0,-1.0,
    -1.0,-1.0,-1.0, -1.0, 1.0, 1.0, -1.0, 1.0,-1.0,
    1.0,-1.0, 1.0, -1.0,-1.0, 1.0, -1.0,-1.0,-1.0,
    -1.0, 1.0, 1.0, -1.0,-1.0, 1.0, 1.0,-1.0, 1.0,
    1.0, 1.0, 1.0, 1.0,-1.0,-1.0, 1.0, 1.0,-1.0,
    1.0,-1.0,-1.0, 1.0, 1.0, 1.0, 1.0,-1.0, 1.0,
    1.0, 1.0, 1.0, 1.0, 1.0,-1.0, -1.0, 1.0,-1.0,
    1.0, 1.0, 1.0, -1.0, 1.0,-1.0, -1.0, 1.0, 1.0,
    1.0, 1.0, 1.0, -1.0, 1.0, 1.0, 1.0,-1.0, 1.0]
uvdata = [
0.000059, 1.0-0.000004, 0.000103, 1.0-0.336048, 
0.335973, 1.0-0.335903, 1.000023, 1.0-0.000013, 
0.667979, 1.0-0.335851, 0.999958, 1.0-0.336064, 
0.667979, 1.0-0.335851, 0.336024, 1.0-0.671877, 
0.667969, 1.0-0.671889, 1.000023, 1.0-0.000013, 
0.668104, 1.0-0.000013, 0.667979, 1.0-0.335851, 
0.000059, 1.0-0.000004, 0.335973, 1.0-0.335903, 
0.336098, 1.0-0.000071, 0.667979, 1.0-0.335851, 
0.335973, 1.0-0.335903, 0.336024, 1.0-0.671877, 
1.000004, 1.0-0.671847, 0.999958, 1.0-0.336064, 
0.667979, 1.0-0.335851, 0.668104, 1.0-0.000013, 
0.335973, 1.0-0.335903, 0.667979, 1.0-0.335851, 
0.335973, 1.0-0.335903, 0.668104, 1.0-0.000013, 
0.336098, 1.0-0.000071, 0.000103, 1.0-0.336048, 
0.000004, 1.0-0.671870, 0.336024, 1.0-0.671877, 
0.000103, 1.0-0.336048, 0.336024, 1.0-0.671877, 
0.335973, 1.0-0.335903, 0.667969, 1.0-0.671889, 
1.000004, 1.0-0.671847, 0.667979, 1.0-0.335851]
gl = GL()
makeWindow(gl, (800, 600), "Test Title")
gl.glfwEnable(gl.GLFW_STICKY_KEYS)
vertexArrayID = gl.glGenVertexArray()
gl.glBindVertexArray(vertexArrayID)
gl.glClearColor(0.0, 0.0, 0.3, 1.0)
gl.glEnable(gl.GL_DEPTH_TEST)
gl.glDepthFunc(gl.GL_LESS) 
programID = LoadShaders(gl, vertexShader, fragmentShader)
vertexPosition_modelspace_loc = gl.glGetAttribLocation(programID, 'vertexPosition_modelspace')
matrixID = gl.glGetUniformLocation(programID, "MVP")
vertexUVID = gl.glGetAttribLocation(programID, "vertexUV")
data = gl.new('GLfloat[]', vdata)
vertexbuffer =  gl.glGenBuffer()
gl.glBindBuffer(gl.GL_ARRAY_BUFFER, vertexbuffer)
gl.glBufferDataf(gl.GL_ARRAY_BUFFER, 
                data, gl.GL_STATIC_DRAW)
uvBuffer = gl.glGenBuffer()
gl.glBindBuffer(gl.GL_ARRAY_BUFFER, uvBuffer)
data2 = gl.new('GLfloat[]', uvdata)
gl.glBufferDataf(gl.GL_ARRAY_BUFFER, 
                data2, gl.GL_STATIC_DRAW)
projectionMatrix = perspective(math.pi/4.0, 4.0 / 3.0, 0.1, 100.0)
viewMatrix = lookat(vec3(4,3,3), vec3(0,0,0), vec3(0,1,0))                      
mvp = viewMatrix * projectionMatrix
texture = gl.glGenTexture()
gl.glBindTexture(gl.GL_TEXTURE_2D, texture)
width, height, data = readPNG('uvtemplate.png')
gl.glTexImage2D(gl.GL_TEXTURE_2D, 0, gl.GL_RGB, width, height, 0, gl.GL_RGB, gl.GL_UNSIGNED_BYTE, data);
gl.glTexParameteri(gl.GL_TEXTURE_2D, gl.GL_TEXTURE_MAG_FILTER, gl.GL_NEAREST);
gl.glTexParameteri(gl.GL_TEXTURE_2D, gl.GL_TEXTURE_MIN_FILTER, gl.GL_NEAREST);
textureID  = gl.glGetUniformLocation(programID, "myTextureSampler");
def tst(x, y):
    print x, y

gl.glfwSetMouseButtonCallback(tst)
while (gl.glfwGetKey(gl.GLFW_KEY_ESC) != gl.GLFW_PRESS and 
                    gl.glfwGetWindowParam(gl.GLFW_OPENED)):
    gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_DEPTH_BUFFER_BIT)
    gl.glUseProgram(programID)
    gl.glUniformMatrix4fv(matrixID, 1, gl.GL_FALSE, mvp.data());
    gl.glActiveTexture(gl.GL_TEXTURE0);
    gl.glBindTexture(gl.GL_TEXTURE_2D, texture)
    gl.glUniform1i(textureID, 0);
    gl.glEnableVertexAttribArray(0);
    gl.glBindBuffer(gl.GL_ARRAY_BUFFER, vertexbuffer);
    gl.glVertexAttribPointer(vertexPosition_modelspace_loc, 3, gl.GL_FLOAT, 
                                 gl.GL_FALSE, 0, gl.offsetPointer(0))
    gl.glEnableVertexAttribArray(1);
    gl.glBindBuffer(gl.GL_ARRAY_BUFFER, uvBuffer);
    gl.glVertexAttribPointer(vertexUVID, 2, gl.GL_FLOAT, 
                             gl.GL_FALSE, 0, gl.offsetPointer(0));
    gl.glDrawArrays(gl.GL_TRIANGLES, 0, 3*12);
    gl.glDisableVertexAttribArray(0);
    gl.glDisableVertexAttribArray(1);
    gl.glfwSwapBuffers()
gl.glfwTerminate()
