"""
PyGL3Display
(C) David Griffin (2010-present)
"Habilain" is a pseudonym used by David Griffin in a number of places.

A high performance Pygame OpenGL library, capable of handling 1000s of sprites

Requirements: Python, Pygame, PyOpenGL, Open GL 2.0 graphics hardware

Licensed under GPLv2 or GPLv3. You should have received a copy of both of these
licenses in the files GPLv2.txt and GPLv3.txt. If not these licenses can be
found at http://www.gnu.org/licenses/gpl-2.0.html (GPLv2) and
http://www.gnu.org/licenses/gpl-3.0.html (GPLv3). By using this file you agree
to be bound by at least one of these licenses.

Other licenses, including commercial/proprietary licenses, can be arranged by 
contacting me via e-mail at habilain@gmail.com. If you do not have in 
writing an alternative license agreement from me, you must use one of the 
licenses specified above when releasing work which makes use of this library.

pygl3display.gl.ctypes

Encapsulates cTypes as used in pygl3display, so that I can easily swap it 
with my own cffi-based cArray later

Usage of this binding -

cdata['c'](10) = 10 chars
cdata['f'](40) = 40 floats etc.. 
"""

import ctypes

cdata = {'c': lambda x: (ctypes.c_char   * x)(),
         'b': lambda x: (ctypes.c_byte   * x)(),
         'B': lambda x: (ctypes.c_ubyte  * x)(),
         'h': lambda x: (ctypes.c_short  * x)(),
         'H': lambda x: (ctypes.c_ushort * x)(),
         'i': lambda x: (ctypes.c_int    * x)(),
         'I': lambda x: (ctypes.c_uint   * x)(),
         'l': lambda x: (ctypes.c_long   * x)(),
         'L': lambda x: (ctypes.c_ulong  * x)(),
         'f': lambda x: (ctypes.c_float  * x)(),
         'd': lambda x: (ctypes.c_double * x)(),
}

cdataSizes = {'c': 1, 'b': 1, 'B': 1, 'h': 2, 'H': 2, 
              'i': 4, 'I': 4, 'l': 8, 'L': 8, 'f': 4, 'd': 8}

def offsetPointer(offset, dtype='B'):
    """Returns a void pointer with a value equal to offset * dtype
    (i.e. offset by dtype items); dtype defaults to bytes"""
    size = cdataSizes[dtype]
    return ctypes.c_void_p(offset * size)

def sizeOf(data):
    return ctypes.sizeof(data)

def byref(data, offset):
    """Corresponds to ctypes byref function, but offset is in 
    sizeof(datatype) rather than bytes"""
    size = int(ctypes.sizeof(data) / len(data))
    return ctypes.byref(data, size*offset)
