"""
PyGL3Display
(C) David Griffin (2010-present)
"Habilain" is a pseudonym used by David Griffin in a number of places.

A high performance Pygame OpenGL library, capable of handling 1000s of sprites

Requirements: Python, Pygame, PyOpenGL, Open GL 2.0 graphics hardware

Licensed under GPLv2 or GPLv3. You should have received a copy of both of these
licenses in the files GPLv2.txt and GPLv3.txt. If not these licenses can be
found at http://www.gnu.org/licenses/gpl-2.0.html (GPLv2) and
http://www.gnu.org/licenses/gpl-3.0.html (GPLv3). By using this file you agree
to be bound by at least one of these licenses.

Other licenses, including commercial/proprietary licenses, can be arranged by 
contacting me via e-mail at habilain@gmail.com. If you do not have in 
writing an alternative license agreement from me, you must use one of the 
licenses specified above when releasing work which makes use of this library.

pygl3display.glcommon.pygamecommon

A GLCommon instance that uses pygame
"""
import pygame
from pygl3display.initialise import Initialiser
from pygl3display.glcommon.base import GLCommonClass


class GLCommonPygame(GLCommonClass):
    @staticmethod
    def createContext(resolution, doubleBuffer, fullscreen):
        """Creates a Pygame OpenGL Window"""
        mode = pygame.OPENGL
        if doubleBuffer: 
            mode |= pygame.DOUBLEBUF
        if fullscreen: 
            mode |= pygame.FULLSCREEN
        pygame.init()
        pygame.display.set_mode(resolution, mode)
        Initialiser.initialise()
        
    redraw = staticmethod(pygame.display.flip)
    init = staticmethod(pygame.init)
    quit = staticmethod(pygame.quit)
