"""
PyGL3Display
(C) David Griffin (2010-present)
"Habilain" is a pseudonym used by David Griffin in a number of places.

A high performance Pygame OpenGL library, capable of handling 1000s of sprites

Requirements: Python, Pygame, PyOpenGL, Open GL 2.0 graphics hardware

Licensed under GPLv2 or GPLv3. You should have received a copy of both of these
licenses in the files GPLv2.txt and GPLv3.txt. If not these licenses can be
found at http://www.gnu.org/licenses/gpl-2.0.html (GPLv2) and
http://www.gnu.org/licenses/gpl-3.0.html (GPLv3). By using this file you agree
to be bound by at least one of these licenses.

Other licenses, including commercial/proprietary licenses, can be arranged by 
contacting me via e-mail at habilain@gmail.com. If you do not have in 
writing an alternative license agreement from me, you must use one of the 
licenses specified above when releasing work which makes use of this library.

pygl3display.sprites

This module provides "oldschool style" 2d Sprite specific features
"""

from pygl3display.sprites.textureatlas import TextureAtlas2d
from pygl3display.sprites.shader import SpriteShader
from pygl3display.sprites.spritecommon import GL3Rect, SpriteCommon
from pygl3display.sprites.image import Surface
from pygl3display.sprites.sprite import (Sprite, 
     topLeftOffsets, topRightOffsets, bottomLeftOffsets, 
     bottomRightOffsets, centerOffsets, makeAdjacentSprites)
from pygl3display.sprites.attribsprite import AttribSprite
from pygl3display.sprites.spritebuffer import SpriteBuffer
from pygl3display.sprites.groups import SpriteGroup

PYGAMEENABLED = False
try:
    import pygame
    del pygame
    PYGAMEENABLED = True
except ImportError:
    pass
PNGENABLED = False
try:
    import png
    del png
    PNGENABLED = True
except ImportError:
    pass
    
if PYGAMEENABLED:
    # Prefer pygame...
    from pygl3display.sprites.pygameimages import loadImage
elif PNGENABLED:
    # ... but fall back on PyPNG
    from pygl3display.sprites.pypngimages import loadImage
